import click
import helium
import util
import timeseries as ts

pass_service=click.make_pass_decorator(helium.Service)

@click.group()
def cli():
    """Operations on physical or virtual sensors.
    """
    pass

def _tabulate(result):
    util.tabulate(result, [
        ('id', util.shorten_json_id),
        ('mac', 'meta/mac'),
        ('name', 'attributes/name'),
    ])


@cli.command()
@click.option('-l', '--label',
              help="the id of a label")
@pass_service
def list(service, label):
    """List sensors.

    Lists information for a label of sensors or all sensors in
    the organization.
    """
    if label:
        label = util.lookup_resource_id(service.get_labels, label)
        sensors = service.get_label(label,include="sensor").get('included')
    else:
        sensors = service.get_sensors().get('data')
    _tabulate(sensors)


@cli.command()
@click.option('--name', required=True,
              help="the name for the new virtual sensor")
@pass_service
def create(service, name):
    """Create a virtual sensor.

    Create a virtual sensor with a name.
    """
    sensors = [service.create_sensor(name=name).get('data')]
    _tabulate(sensors)

@cli.command()
@click.argument('sensor')
@click.option('--name',
              help="the new name for the sensor")
@pass_service
def update(service, sensor, **kwargs):
    """Updates the attributes of a sensor.

    Updates the attributes of a given SENSOR.
    """
    sensor = util.lookup_resource_id(service.get_sensors, sensor)
    data = service.update_sensor(sensor, **kwargs).get('data')
    _tabulate([data])


@cli.command()
@click.argument('sensor')
@pass_service
def delete(service, sensor):
    """Delete a sensor.

    Deletes the SENSOR with the given id.
    """
    sensor = util.lookup_resource_id(service.get_sensors, sensor)
    result = service.delete_sensor(sensor)
    click.echo("Deleted" if result.status_code == 204 else result)


@cli.command()
@click.argument('sensor')
@ts.options()
@pass_service
def timeseries(service, sensor, **kwargs):
    """List readings for a sensor.

    Lists one page of readings for a given SENSOR.
    Readings can be filtered by PORT and by START and END date. Dates are given
    in ISO-8601 and may be one of the following forms:

    \b
    * YYYY-MM-DD - Example: 2016-05-05
    * YYYY-MM-DDTHH:MM:SSZ - Example: 2016-04-07T19:12:06Z

    """
    sensor = util.lookup_resource_id(service.get_sensors, sensor)
    data = service.get_sensor_timeseries(sensor, **kwargs).get('data')
    ts.tabulate(data)


@cli.command()
@ts.format_option()
@click.argument('sensor')
@ts.options(page_size=5000)
@pass_service
def dump(service, sensor, format, **kwargs):
    """Dumps timeseries data to files.

    Dumps the timeseries data for one SENSOR to a file.
    If no sensors or label is specified all sensors for the organization are dumped.

    One file is generated with the sensor id as filename and the
    file extension based on the requested dump format

    This command takes the same arguments as the `timeseries` command, including
    the ability to filter by PORT, START and END date
    """
    sensor = util.lookup_resource_id(service.get_sensors, sensor)
    ts.dump(service, [sensor], format, **kwargs)



from collections import OrderedDict
import dpath.util as dpath
import sys
import click
import uuid
import json
import writer
import urlparse


def is_uuid(str):
    try:
        uuid.UUID(str)
        return True
    except ValueError:
        return False

def lookup_resource_id(list, id_rep):
    if hasattr(list, '__call__'):
        list = list().get('data')
    lookup = {}
    _is_uuid = is_uuid(id_rep)
    short_id_matches = []
    name_matches = []
    for entry in list:
        entry_id = entry.get('id')
        if _is_uuid:
            if entry_id == id_rep:
                return entry_id
        else:
            short_id = shorten_id(entry_id)
            if short_id == id_rep:
                short_id_matches.append(entry_id)
            else:
                try:
                    entry_name = dpath.get(entry, "attributes/name")
                    if entry_name == id_rep:
                        name_matches.append(entry_id)
                except KeyError:
                    pass
    if len(short_id_matches) == 0 and len(name_matches) == 0:
        raise KeyError('Id ' + id_rep.encode('ascii')  + ' does not exist')
    elif len(short_id_matches) == 1 and len(name_matches) == 0:
        return short_id_matches[0]
    elif len(short_id_matches) == 0 and len(name_matches) == 1:
        return name_matches[0]
    else:
        raise KeyError('Ambiguous id: ' + id_rep.encode('ascii'))

def shorten_id(str):
    return str.split('-')[0]

def shorten_json_id(json):
    # Ugh, reaching for global state isn't great but very convenient here
    shorten = not click.get_current_context().find_root().params.get('uuid', False)
    json_id = json.get('id')
    return shorten_id(json_id) if shorten else json_id


def tabulate(result, map):
    mapping = OrderedDict(map)
    if not mapping or not result: return result

    # Ugh, reaching for global state isn't great but very convenient here
    default_format = 'tty' if sys.stdout.isatty() else 'csv'
    format = click.get_current_context().find_root().params.get('format')
    # Ensure format is set to something sensible
    format = format or default_format
    file = click.utils.get_text_stream('stdout')
    output = writer.for_format(format, file, mapping=mapping)
    output.start()
    output.write_entries(result)
    output.finish()

def map_script_filenames(json):
    files = dpath.get(json, 'meta/scripts')
    return ', '.join(extract_script_filenames(files))

def extract_script_filenames(files):
    return [urlparse.urlsplit(url).path.split('/')[-1] for url in files]



import click
import util
import helium

pass_service=click.make_pass_decorator(helium.Service)

@click.group()
def cli():
    """Operations on elements.
    """
    pass

def _tabulate(result):
    util.tabulate(result, [
        ('id', util.shorten_json_id),
        ('mac', 'meta/mac'),
        ('name', 'attributes/name')
    ])


@cli.command()
@click.argument('element',required=False)
@pass_service
def list(service, element):
    """List elements.

    Lists one or all elements in the organization.
    """
    if element:
        element = util.lookup_resource_id(service.get_elements, element)
        elements=[service.get_element(element).get('data')]
    else:
        elements=service.get_elements().get('data')
    _tabulate(elements)


@cli.command()
@click.argument('element')
@click.option('--name',
              help="the new name for the sensor")
@pass_service
def update(service, element, **kwargs):
    """Updates the attributes of an element.

    Updates the attributes of a given ELEMENT.
    """
    sensor = util.lookup_resource_id(service.get_elements, element)
    data = service.update_element(sensor, **kwargs).get('data')
    _tabulate([data])
